module Pod
  class Command
    class Repo < Command
      class AddCodingAr < Repo
        self.summary = 'Add a CODING-AR-backed repo'
        self.description = <<-DESC
          Adds the remote named `NAME` to the local spec-repos directory at `#{Config.instance.repos_dir}`.
        DESC

        def initialize(argv)
          @name, @url = argv.shift_argument, argv.shift_argument
          super
        end

        def validate!
          super
          unless @name && @url
            help! 'This command requires both a repo name and a url.'
          end
        end

        def run
          require 'coding_ar_util'

          UI.section("Adding CODING-AR-backed repository `#@url` into local spec repo `#@name`") do
            if !CodingArUtil.coding_ar_service?(@url)
              raise Informative, "`#@url` seems not to be a CODING-AR-backed repository."
            end

            raise Informative, "Local spec repo #@name (in #{dir}) already exists." if File.exists?(dir)

            FileUtils.mkdir_p dir

            begin
              url_path = create_coding_ar_url_file dir
            rescue => e
              raise Informative, "Cannot create file '#{url_path}' because : #{e.message}."
            end
          end
        end

        def create_coding_ar_url_file(repo_dir)
          url_path = File.join(repo_dir, ".coding_ar_url")
          url_file = File.new(url_path, "wb")
          url_file << @url
          url_file.close
          url_path
        end
      end
    end
  end
end
  
module Pod
  class Command
    class Repo < Command
      class PushCodingAr < Repo
        extend Executable
        executable :tar

        self.summary = 'Push a pod to CODING-AR.'
        self.description = <<-DESC
          Push a pod include its spec to a CODINF-AR repo.
        DESC

        self.arguments = [
            CLAide::Argument.new('REPO', true),
            CLAide::Argument.new('NAME.podspec', true)
        ]

        def initialize(argv)
          @repo = argv.shift_argument
          @podspec = argv.shift_argument
          super
        end

        def validate!
          super
          unless @repo && @podspec
            help! 'This command requires both a repo and a podspec.'
          end
        end

        def run
          unless File.file? File.join(Config.instance.repos_dir, @repo, ".coding_ar_url")
            raise Informative, "`#@repo` is not a CODING-AR repo.".red
          end

          FileUtils.mkdir_p tmp_dir
          begin
            UI.section("Pushing pod `#{spec.name} #{spec.version.to_s}`` to CODING-AR repo `#@repo`") do
              pod_tar_gz_path = create_pod_tar_gz
              podspec_json_path = create_spec_json

              post_pod_with_spec(pod_tar_gz_path, podspec_json_path)
            end
            UI.puts "Successfully push pod `#{spec.name} #{spec.version.to_s}`` to CODING-AR repo `#@repo`".green
          ensure
            FileUtils.rm_rf tmp_dir
          end
        end

        def create_pod_tar_gz
          require 'digest/sha1'
          require 'find'
          
          pod_dir = File.join(tmp_dir, "pod")
          FileUtils.mkdir_p(pod_dir)

          pod_root = File.dirname Pathname(@podspec).realpath
          path_list = Pod::Sandbox::PathList.new(pod_root)

          file_accessors ||= spec.available_platforms.flat_map do |platform| 
            Pod::Sandbox::FileAccessor.new(path_list, spec.consumer(platform))
          end

          pod_files = Pod::Sandbox::FileAccessor.all_files(file_accessors).map do |path|
            Find.find(path).select { |path| File.file? path}
          end

          pod_files.flatten.each do |source|
            target = File.join(pod_dir, Pathname(source).relative_path_from(pod_root))
            FileUtils.mkdir_p(File.dirname(target))
            FileUtils.copy_file(source, target)
          end

          tar_gz_path = File.join(tmp_dir, "#{spec.name}-#{spec.version}.tar.gz")

          Dir.chdir(pod_dir) { tar!('-czf', tar_gz_path, '.') }

          tar_gz_path
        end

        def create_spec_json
          require 'json'

          FileUtils.mkdir_p(tmp_dir)
          podspec_json_path = File.join(tmp_dir, "#{spec.name}.podspec.json")
          podspec_json = File.new(podspec_json_path, "wb")
          podspec_json.puts(spec.to_pretty_json)
          podspec_json.close
          podspec_json_path
        end

        def post_pod_with_spec(pod_path, spec_path)
          require 'json'

          request_body = {
            'pod'   => file_push_profile(pod_path),
            'spec'  => file_push_profile(spec_path)
          }
          request_body_json_path = File.join(tmp_dir, "request_body.json")
          request_body_json = File.new(request_body_json_path, "wb")
          request_body_json.puts(request_body.to_json)
          request_body_json.close

          require 'coding_ar_util'
          CodingArUtil.push_pod(coding_ar_repo_url, request_body_json_path)
        end

        def spec
          spec_file = Pathname(@podspec)
          spec = Pod::Specification.from_file(spec_file)
        end

        def tmp_dir
          File.join(Config.instance.repos_dir, @repo, '.tmp', spec.name, spec.version.to_s)
        end

        def coding_ar_repo_url
          File.read File.join(Config.instance.repos_dir, @repo, ".coding_ar_url")
        end

        def file_push_profile(path)
          require 'digest/sha1'

          content = File.read(path)
          {
            'size'            => File.lstat(path).size,
            'base64_content'  => Base64.strict_encode64(content),
            'sha1_digest'     => Digest::SHA1.hexdigest(content)
          }
        end
      end
    end
  end
end